/*
*       This is the Raiden cipher source code.
*       This file includes the encoding and decoding routines
*
*       Developed by Javier Polimn and Julio Cesar Hernandez
*       Updated by A.Myasnikov to Raiden-256 with 256 bit key
*       Last update: 29th of Jan, 2009.
*
*       Web: www.darksoftware.narod.ru
*/


#include <mem.h>;

typedef unsigned long u32;

void __export __stdcall
crypt (u32 * data, u32 * result, u32 * key)
{

  u32 b0 = data[0];
  u32 b1 = data[1];
  u32 i;
  u32 sk1, sk2;
  u32 k1[4], k2[4];
  memcpy (&k1[0], &key[0], 16);
  memcpy (&k2[0], &key[4], 16);


  for (i = 0; i != 16; i++)
    {
      sk1 = k1[i % 4] =
	((k1[0] + k1[1]) + ((k1[2] + k1[3]) ^ (k1[0] << k1[2])));
      sk2 = k2[i % 4] =
	((k2[0] + k2[1]) + ((k2[2] + k2[3]) ^ (k2[0] << k2[2])));
      b0 += ((sk1 + b1) << 9) ^ ((sk1 - b1) ^ ((sk1 + b1) >> 14));
      b1 += ((sk2 + b0) << 9) ^ ((sk2 - b0) ^ ((sk2 + b0) >> 14));
    }
  result[0] = b0;
  result[1] = b1;
}


void __export __stdcall
decrypt (u32 * data, u32 * result, u32 * key)
{
  u32 b0 = data[0];
  u32 b1 = data[1];
  u32 i;
  u32 k1[4], k2[4];
  u32 sk1[16], sk2[16];
  memcpy (&k1[0], &key[0], 16);
  memcpy (&k2[0], &key[4], 16);

  for (i = 0; i != 16; i++)
    {
      k1[i % 4] = ((k1[0] + k1[1]) + ((k1[2] + k1[3]) ^ (k1[0] << k1[2])));
      sk1[i] = k1[i % 4];
      k2[i % 4] = ((k2[0] + k2[1]) + ((k2[2] + k2[3]) ^ (k2[0] << k2[2])));
      sk2[i] = k2[i % 4];

    }
  for (i = 15; i != -1; i--)
    {

      b1 -= ((sk2[i] + b0) << 9) ^ ((sk2[i] - b0) ^ ((sk2[i] + b0) >> 14));
      b0 -= ((sk1[i] + b1) << 9) ^ ((sk1[i] - b1) ^ ((sk1[i] + b1) >> 14));
    }
  result[0] = b0;
  result[1] = b1;

}
